=== modified file '__init__.py'
--- __init__.py	2005-08-31 18:52:32 +0000
+++ __init__.py	2005-09-01 17:03:54 +0000
@@ -31,6 +31,7 @@
         clean_tree.clean_tree(ignored=ignored)
 
 bzrlib.commands.OPTIONS['no-collapse'] = None
+bzrlib.commands.OPTIONS['no-antialias'] = None
 
 class cmd_graph_ancestry(bzrlib.commands.Command):
     """Produce ancestry graphs using dot.
@@ -46,12 +47,16 @@
     the revno.  Other revisions are labeled with essentially random numbers.
 
     Revisions that are not in branch storage have dotted outlines.
+
+    rsvg is used to antialias PNG and JPEG output, but this can be disabled
+    with --no-antialias.
     """
     takes_args = ['branch', 'file']
-    takes_options = ['no-collapse']
-    def run(self, branch, file, no_collapse=False):
+    takes_options = ['no-collapse', 'no-antialias']
+    def run(self, branch, file, no_collapse=False, no_antialias=False):
         import graph
-        graph.write_ancestry_file(branch, file, not no_collapse)
+        graph.write_ancestry_file(branch, file, not no_collapse, 
+                                  not no_antialias)
 
 commands = [push.cmd_push, annotate.cmd_annotate, shelf.cmd_shelve, 
             shelf.cmd_unshelve, cmd_clean_tree, conflicts.cmd_conflicts,

=== modified file 'dotgraph.py'
--- dotgraph.py	2005-09-01 16:06:54 +0000
+++ dotgraph.py	2005-09-01 17:03:54 +0000
@@ -4,6 +4,8 @@
 from xml.sax.saxutils import escape
 import os.path
 import errno
+import tempfile
+import shutil
 
 mail_map = {}
 
@@ -11,6 +13,10 @@
     def __init__(self):
         Exception.__init__(self, "Can't find dot!")
 
+class NoRsvg(Exception):
+    def __init__(self):
+        Exception.__init__(self, "Can't find rsvg!")
+
 class Node(object):
     def __init__(self, name, color=None, label=None, rev_id=None,
                  cluster=None):
@@ -108,6 +114,25 @@
             yield "}\n"
     yield "}\n"
 
+def invoke_dot_aa(input, out_file, file_type='png'):
+    """\
+    Produce antialiased Dot output, invoking rsvg on an intermediate file.
+    rsvg only supports png, jpeg and .ico files."""
+    tempdir = tempfile.mkdtemp()
+    try:
+        temp_file = os.path.join(tempdir, 'temp.svg')
+        invoke_dot(input, temp_file, 'svg')
+        cmdline = ['rsvg', temp_file, out_file]
+        try:
+            rsvg_proc = Popen(cmdline)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                raise NoRsvg()
+        status = rsvg_proc.wait()
+    finally:
+        shutil.rmtree(tempdir)
+    return status
+
 def invoke_dot(input, out_file=None, file_type='svg', antialias=None):
     cmdline = ['dot', '-T%s' % file_type]
     if out_file is not None:

=== modified file 'graph.py'
--- graph.py	2005-09-01 16:06:54 +0000
+++ graph.py	2005-09-01 17:03:54 +0000
@@ -1,4 +1,5 @@
-from dotgraph import Node, dot_output, invoke_dot, NoDot, mail_map
+from dotgraph import Node, dot_output, invoke_dot, invoke_dot_aa, NoDot 
+from dotgraph import mail_map
 from bzrlib.branch import Branch
 from bzrlib.errors import BzrCommandError
 import bzrlib.errors
@@ -105,16 +106,25 @@
             node_relations.append((nodes[key], nodes[value]))
     return node_relations
 
-def write_ancestry_file(branch, filename, collapse=True):
+def write_ancestry_file(branch, filename, collapse=True, antialias=True):
     b = Branch(branch)
     relations = graph_ancestry(b, collapse)
     ext = filename.split('.')[-1]
-    if ext in ('svg', 'svgz', 'gif', 'jpg', 'ps', 'fig', 'mif', 'png'):
+    if antialias and ext in ('png', 'jpg'):
+        try:
+            invoke_dot_aa(dot_output(relations), filename, ext)
+        except NoDot, e:
+            raise BzrCommandError("Can't find 'dot'.  Please ensure Graphviz"\
+                " is installed correctly.")
+        except NoRsvg, e:
+            raise BzrCommandError("Can't find 'rsvg'.  Please ensure "\
+                "librsvg-bin is installed correctly, or use --noantialias.")
+    elif ext in ('svg', 'svgz', 'gif', 'jpg', 'ps', 'fig', 'mif', 'png'):
         try:
             invoke_dot(dot_output(relations), filename, ext)
         except NoDot, e:
             raise BzrCommandError("Can't find 'dot'.  Please ensure Graphviz"\
-                " is installed correctly.")
+                " is installed correctly, or use --noantialias")
     elif ext=='dot':
         file(filename, 'wb').write("".join(list(dot_output(relations))))
     else:

